#include "LL_USB_min.h"
#include "stdio.h"
#include <termios.h>

extern SCAN_STRUCT scanStruct;	

USHORT devID;
 
BOOLEAN setDefaultVars(void);
BOOLEAN Scan(void);

BOOLEAN chkKbHit(void);
int doGetch(void);

int main(void)
{
	devID = 5206;
	double dblRate = 100;
	BYTE bChan = 0;
	setDefaultVars();

	system ("clear");

	printf("\n calling EX_ConnectOneDevice with rate: %0.4lf, chan: %d \n", dblRate, bChan);   
	if(!EX_ConnectOneDevice (devID, dblRate, bChan))
	{
		printf("\nfailed call to EX_ConnectOneDevice\n\n");
		return 0;
	}

	printf("\n calling EX_GetOneConversion chan: %d \n", bChan);   
	double dblMilliVolts;
	if(!EX_GetOneConversion (devID, &dblMilliVolts))
	{
		printf("\nfailed call to EX_GetOneConversion\n\n");
		return 0;
	}
	printf(" voltage: %lf\n", dblMilliVolts);


	bChan = 6;
	printf("\n calling EX_SendChan with chan: %d \n", bChan);   

	BYTE bLastDigin = 0;
	if(!EX_SendChan(devID, bChan, &bLastDigin))
	{
		printf("\nfailed call to EX_SendChan\n\n");
		return 0;
	}

	printf(" calling EX_GetOneConversion for channel change check \n");   
	if(!EX_GetOneConversion (devID, &dblMilliVolts))
	{
		printf("\nfailed call to EX_GetOneConversion\n\n");
		return 0;
	}
	printf(" voltage check for channel %d: %lf\n\n", bChan, dblMilliVolts);

	BYTE bDAC = 0;
	double dblVoltage = 3.5;
	printf(" calling EX_SendDAC. Setting DAC: %d, to volts: %lf \n", bDAC, dblVoltage);   
	if(!EX_SendDAC(devID, dblVoltage, bDAC))
	{
		printf("\nfailed call to EX_SendDAC\n\n");
		return 0;
	}
	bChan = 3;
	if(!EX_SendChan(devID, bChan, &bLastDigin))
	{
		printf("\nfailed call to EX_SendChan\n\n");
		return 0;
	}
	printf(" calling EX_GetOneConversion for DAC check with DAC 0 connected to chan 3 \n"); 
	if(!EX_GetOneConversion (devID, &dblMilliVolts))
	{
		printf("\nfailed call to EX_GetOneConversion\n\n");
		return 0;
	}
	printf(" voltage check for channel %d: %lf\n\n", bChan, dblMilliVolts);

	dblRate = 1000;
	double dblRateOriginal = dblRate;
	printf(" calling EX_GetCalculatedRate for \"actual rate\" for %0.2lf \n", dblRate); 
	if(!EX_GetCalculatedRate (devID, &dblRate))
	{
		printf("\nfailed call to EX_GetCalculatedRate\n\n");
		return 0;
	}
	printf(" \"actual rate\" for %.4lfHz.: %.5lfHz. \n\n", dblRateOriginal, dblRate); 

	printf(" starting scan - press any key to stop scan\n"); 
	Scan();

    return 1;
}


//##########################
//##########################
//### setDefaultVars #######
//##########################
//##########################
BOOLEAN setDefaultVars(void)
{
	//NOTE: default var vals must be set here to share common header
	// could set these in a varSet() type function
	fCurrentScanTypeChanged = FALSE;
	dblRate = 100;
	bSingleChanScanChan = 0;
	fSignedOn = FALSE;
	fGotDigin = FALSE;
	bLastDigin = 0;
	fDoScanLog = FALSE;
	fDoDispData = TRUE; 

	usDeviceID = 0;
	iErr = 0;
	bDBG = 0;
	fContScanDone = FALSE;		
	fScanThreadActive = FALSE;	
}



//##############
//##############
//### Scan #####
//##############
//##############
BOOLEAN Scan(void)
{
//constants shown just for reference
/*
#define CMND_SINGLE_CHAN_SCAN			0x89	// single chan regular
#define CMND_SINGLE_CHAN_DIGIN_SCAN		0x8D	// single chan with digin
#define CMND_MULTI_CHAN_SCAN			0x8B	// multi chan regular
#define CMND_MULTI_CHAN_CAL_SCAN		0x8C	// multi chan with cal
#define CMND_MULTI_CHAN_DIGIN_SCAN		0x8E	// multi chan with digin
#define CMND_MULTI_CHAN_DIGIN_CAL_SCAN	0x8F	// multi chan with cal and digin
*/
	fUSB_ERROR = FALSE; 

	int err;
	scanStruct.inThread = 1;
	int iVal = 23;

	scanStruct.uiBufferSizeMin = 500; 

	fContScanDone = FALSE; 
	fMainScanLoopBusy = TRUE; 
	fEndScanTokenSent = FALSE; 


	scanStruct.uiBufferSizeMax;
	scanStruct.uiHeadPntr = 0;
	scanStruct.uiTailPntr = 0;

	scanStruct.isScanning = 0;
	scanStruct.fGotBuffWrap = 0; 
	scanStruct.fScanThreadLoopDone = FALSE;  
	fScanThreadActive = FALSE; //4-20-17 replace other flags with this one

	//following to flag it locally so a message can be displayed by way of this 
	// function at an appropriate place in the exit screen rather than disturing
	// voltage display if flagging global scanStruct.fGotBuffWrap which could cause
	// a disruption in the dislay flow by helperFuncs.c
	BOOLEAN fGotScanErr_TEMP = FALSE; 


	//following just defaults, will be asked for both when the scan is started
	//search "doSetNumberOfScans" for explanation using each of the following
	fContinuousScan = FALSE;	//this trumps the others. If true, uiScansToDo is ignored
	fTimedScan = FALSE;
	uiScansToDo = 100;		//"scans" not data points (default to 100)
	uiScanHours = 1;		//just any default
	uiScanMinutes = 1;		//just any default 
	uiScanSeconds = 60;		//just any default 
	fScanHours = FALSE;		//just any default 
	fScanMinutes = FALSE;	//just any default 
	fScanSeconds = FALSE;	//just any default 
	fContinuousScan = TRUE;
	fDoDispData = TRUE;

	scanStruct.uiScanToken = CMND_SINGLE_CHAN_SCAN;
	scanStruct.uiScanChans = 1;
	scanStruct.fGotBuffWrap = FALSE;
	scanStruct.fScanThreadLoopDone = FALSE;
	scanStruct.isScanning = FALSE; 
	scanStruct.uiStackedScans = 1; //MUST ALWAYS BE AT LEAST 1
	scanStruct.uiHeadPntr = 0;
	scanStruct.uiTailPntr = 0;




	//FOR REFERENCE:
	//#define	CMND_SINGLE_CHAN_SCAN			0x89  /* SingleChan scan */        
	//#define	CMND_MULTI_CHAN_SCAN			0x8B  /* MultiChan scan */
	//#define	CMND_MULTI_CHAN_CAL_SCAN		0x8C  /* MultiCalScan */
	//#define	CMND_SINGLE_CHAN_DIGIN_SCAN		0x8D  /* SingleChanScanning /w digIn */
	//#define	CMND_MULTI_CHAN_DIGIN_SCAN		0x8E  /* MultiScan w/ digIn */
	//#define	CMND_MULTI_CHAN_CAL_DIGIN_SCAN	0x8F  /* MultiCalScan w/ digIn */
	//#define	EndScan							0x8A  /* End a scan */          

	BOOLEAN fIsDigScan = FALSE;
	if((scanStruct.uiScanToken == CMND_SINGLE_CHAN_DIGIN_SCAN)
			|| (scanStruct.uiScanToken == CMND_MULTI_CHAN_DIGIN_SCAN)
			|| (scanStruct.uiScanToken == CMND_MULTI_CHAN_CAL_DIGIN_SCAN))
		fIsDigScan = TRUE;

	BYTE bDigOutIncVal = 14; 
	BYTE bAddForOffsetChan = 0;
	if((scanStruct.uiScanToken == CMND_MULTI_CHAN_CAL_SCAN)
			|| (scanStruct.uiScanToken == CMND_MULTI_CHAN_CAL_DIGIN_SCAN))
	bAddForOffsetChan = 1;

	BYTE bIsSingleChan = 0;
	if((scanStruct.uiScanToken == CMND_SINGLE_CHAN_SCAN) 
			|| (scanStruct.uiScanToken == CMND_SINGLE_CHAN_DIGIN_SCAN))
		bIsSingleChan = 1;

	scanStruct.scanBuff = NULL; //5-9-17 always before allocating/assigning


	//need to allocate in calling code, since thread exits before this does
	// and it will need to be deallocated.
	scanStruct.uiBufferSizeMax = (scanStruct.uiBufferSizeMin * scanStruct.uiStackedScans);
	scanStruct.scanBuff = (INT *)malloc(scanStruct.uiBufferSizeMax * sizeof(INT));

	scanStruct.uiHeadPntr = 0; scanStruct.uiTailPntr = 0;

	if(scanStruct.scanBuff == NULL)
	{
		printf("Failed scanBuff memory allocation\n");
		scanStruct.inThread = 0;
		fMainScanLoopBusy = FALSE;
		return;
	}
	memset(scanStruct.scanBuff, 0, scanStruct.uiBufferSizeMax * sizeof(INT));

	BOOLEAN fIsDiginScan = FALSE;

	scanStruct.diginBuff = NULL; 

	if((scanStruct.uiScanToken == CMND_SINGLE_CHAN_DIGIN_SCAN) 
		 || (scanStruct.uiScanToken == CMND_MULTI_CHAN_DIGIN_SCAN)
		 || (scanStruct.uiScanToken == CMND_MULTI_CHAN_CAL_DIGIN_SCAN))
	{
		fIsDiginScan = TRUE;
		scanStruct.diginBuff = (BYTE *)malloc(scanStruct.uiBufferSizeMax * sizeof(BYTE));
		if(scanStruct.diginBuff == NULL)
		{
			free(scanStruct.scanBuff);
			printf("Failed diginBuff memory allocation\n");
			scanStruct.inThread = 0;
			fMainScanLoopBusy = FALSE;
			return;
		}
		printf("successful diginBuff memory allocation\n");
		memset(scanStruct.diginBuff, 0, scanStruct.uiBufferSizeMax * sizeof(BYTE));
	}


//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
//set thread priority - can be adjusted to lower level if
// there are never any buffer wraps (either firmware or local)
//http://man7.org/linux/man-pages/man7/sched.7.html
//http://stackoverflow.com/questions/13964951/unable-to-set-pthread-priority
	pthread_attr_t *tattr;
	struct sched_param param;
	int policy, prio;
	prio = 99;   //1-99
	tattr = (pthread_attr_t *)malloc(sizeof(pthread_attr_t));
	err = pthread_attr_init(tattr);

	err = pthread_attr_getschedparam(tattr,&param);
	//printf("getschedparam: %u: \n", error);

	err = pthread_attr_setinheritsched(tattr, PTHREAD_EXPLICIT_SCHED);
	//printf("setinheritsched: %u: \n", error);

	policy = SCHED_RR;
	err = pthread_attr_setschedpolicy(tattr, policy);
	//printf("setschedpolicy: %u: \n", error);

	param.sched_priority = prio;
	err = pthread_attr_setschedparam(tattr ,&param);
	//printf("setschedparam: %u: \n", error);

	//error = pthread_create(&myThread,tattr,threadFunc,&numOfExecutions);
	err = pthread_create(&threadID, tattr, &doScan, &scanStruct);
	//printf("create: %u: \n", error);

	pthread_getschedparam(threadID,&policy,&param);
	//printf("policy:: %d pri :: %d\n",policy,param.sched_priority);
//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


	FILE *fpFile = NULL; 
	BYTE bTchan = scanStruct.uiScanChans;	//just to make code less crowded below
	UINT uiSToken = scanStruct.uiScanToken; //just to make code less crowded below

	int cInput; //for use with doGetch();

	UINT uiScanNum = 0;
	if(err == 0)
	{
		char fBuff[512];
		char fBuffSub[128];
		char timeBuff[128];
		FILE *fpFile = NULL;
		BYTE bLogFileCnt = 0; 
		#define FILE_WRITE fwrite(fBuff, 1,  strlen(fBuff), fpFile); fflush(fpFile);
		if(fDoScanLog)
		{
			//time code from http://stackoverflow.com/questions/2242963/get-the-current-time-in-seconds 
			time_t now;
			struct tm *tm;
			BOOLEAN fProceedWithFileCreate = FALSE;

			now = time(0);
			if ((tm = localtime (&now)) == NULL) { }
			else fProceedWithFileCreate = TRUE;
			if(fProceedWithFileCreate)
			{
				sprintf (timeBuff, "%04d-%02d-%02d_%02d-%02d-%02d",
					tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday,
					tm->tm_hour, tm->tm_min, tm->tm_sec);
				char miniBuff[255];
				char miniBuff2[25];
				strcpy(miniBuff, timeBuff);
				sprintf(miniBuff2, "_%u.txt", bLogFileCnt);
				strcat(miniBuff, miniBuff2);
				fpFile = fopen(miniBuff, "w+");

			}
			if(fpFile)
			{
				time_t lt; time( &lt ); 
				sprintf(fBuff, "This log file created:  %s\n", ctime(&lt)); FILE_WRITE
			}
			else fDoScanLog = FALSE;
		}
		else
		{
/* just for testing
printf("\nfDoScanLog = FALSE. Any key to continue\n"); 
while(chkKbHit() == TRUE) 
{ usleep(10); doGetch();}
cInput = doGetch();
printf("\nfDoScanLog = FALSE. Any key (again) to continue\n"); 
while(chkKbHit() == TRUE) 
{ usleep(10); doGetch();}
cInput = doGetch();
*/
		}
		if(fpFile)
		{
			sleep(1);	
			time_t lt; time( &lt ); sprintf(fBuff, "This log file created:  %s\n", ctime(&lt)); 
			FILE_WRITE

			//FOR REFERENCE:
			//#define	CMND_SINGLE_CHAN_SCAN			0x89  /* Start a SingleChan scan */        
			//#define	CMND_MULTI_CHAN_SCAN			0x8B  /* MultiChan scan */
			//#define	CMND_MULTI_CHAN_CAL_SCAN		0x8C  /* MultiCalScan */
			//#define	CMND_SINGLE_CHAN_DIGIN_SCAN		0x8D  /* SingleChanScanning /w digIn */
			//#define	CMND_MULTI_CHAN_DIGIN_SCAN		0x8E  /* MultiScan w/ digIn */
			//#define	CMND_MULTI_CHAN_CAL_DIGIN_SCAN	0x8F  /* MultiCalScan w/ digIn */
			
			if(bIsSingleChan)
			{
				sprintf(fBuff, "SCAN#\t\tchan %d\n", scanStruct.uiScanChans);
				if(scanStruct.uiScanToken == CMND_SINGLE_CHAN_DIGIN_SCAN) 
					strcat(fBuff, "   digin");
				FILE_WRITE
			}

//##########################################################################
//##########################################################################
//##  NOTE #################################################################
//##  will need to replace spaces, with tabs in logfile for spreadsheet-read
//##########################################################################
//##########################################################################
			else
			{
				sprintf(fBuff, "SCAN#\t");
				BYTE b = 0;
				for(b = 0; b < bTchan; b++)
				{
					if(b == 0)
					{
						if(fIsDigScan)sprintf(fBuffSub, "\t chan%d      ", b);
						else sprintf(fBuffSub, "\t chan%d     ", b);
					}
					else 
					{
						if(fIsDigScan)sprintf(fBuffSub, "\t chan%d       ", b);
						else sprintf(fBuffSub, "\t chan%d   ", b);
					}
					strcat(fBuff, fBuffSub);
				}
				if(bAddForOffsetChan) strcat(fBuff, "\t offset");
				strcat(fBuff, "\n");
				FILE_WRITE
			}
		}
		else if(fDoScanLog && (fpFile == NULL))
		{
			printf("\nfile open failed. Any key to continue\n"); 
			while(chkKbHit() == TRUE) 
			{ usleep(10); doGetch();}
			cInput = doGetch();
		}

		int myCnt = 0;
		printf("successful scan startup \n");

//####################################		
//####################################		
//need a timeout in the following loop
//####################################		
//####################################		
		while(1 == 1)
		{ 
			if((scanStruct.inThread == 0) || (scanStruct.fScanThreadLoopDone == TRUE) || (scanStruct.isScanning == 1)) 
				break; 
		}

		if(bIsSingleChan && fDoDispData)  //just 8 across for my display type to fill it up horizontally
		{
			printf("  chan%d        chan%d        chan%d        chan%d        chan%d        chan%d        chan%d",
				bSingleChanScanChan, bSingleChanScanChan, bSingleChanScanChan, bSingleChanScanChan,
				 bSingleChanScanChan,bSingleChanScanChan, bSingleChanScanChan);
			if(fIsDigScan) printf("       digin");
          printf("\n");
		}
		else if (fDoDispData)
		{
			sprintf(fBuff, "SCAN#");
			BYTE b = 0;
			for(b = 0; b < bTchan; b++)
			{
				if(b == 0)
				{
					if(fIsDigScan)sprintf(fBuffSub, "   chan%d    ", b);
					else sprintf(fBuffSub, "    chan%d", b);
				}
				else 
				{
					if(fIsDigScan)sprintf(fBuffSub, "     chan%d    ", b);
					else sprintf(fBuffSub, "     chan%d", b);
				}
				strcat(fBuff, fBuffSub);
			}
			if(bAddForOffsetChan) strcat(fBuff, "     offset");
			strcat(fBuff, "\n");
			printf("%s",fBuff);
		}

		BOOLEAN fBreakAllLoops = FALSE;


		BYTE bCurrentIndexForCalScan = 0;		//component of double buffering (see below)
		DOUBLE dblTempBuffForMultiChanScan [8]; //"double buffering" data aids ease of dealing with 
		
		// cal scan chan when doing a cal scan
		while(scanStruct.inThread == 1)
		{
			if(scanStruct.fScanThreadLoopDone) 
			{
				break; //4-20-17  set if scan thread loop has exited 
			}
			if(scanStruct.isScanning == 1) //may take a while for this flag to be set
			{
				if(scanStruct.uiPointsInBuff > (bTchan + bAddForOffsetChan)) //5-1-17
				{
					bCurrentIndexForCalScan = 0;
					for(bCurrentIndexForCalScan = 0; bCurrentIndexForCalScan 
						< (bTchan + bAddForOffsetChan); bCurrentIndexForCalScan++)
					{
						dblTempBuffForMultiChanScan[bCurrentIndexForCalScan] 
							= (scanStruct.scanBuff[scanStruct.uiTailPntr] - 8388608) * 0.00000059604645;
						scanStruct.uiPointsInBuff = scanStruct.uiPointsInBuff - 1;
						scanStruct.uiTailPntr = scanStruct.uiTailPntr + 1;
						if(scanStruct.uiTailPntr == scanStruct.uiBufferSizeMax) scanStruct.uiTailPntr = 0;
					}

					if(bIsSingleChan)
					{
						for(bCurrentIndexForCalScan = 0; bCurrentIndexForCalScan 
							< (bTchan + bAddForOffsetChan); bCurrentIndexForCalScan++)
						{
							dblVolts = dblTempBuffForMultiChanScan[bCurrentIndexForCalScan];
					
							BOOLEAN fNewLineStuff = FALSE;
							//just for "keeping it on display area" while also showing as many columns as possible
							if(fDoDispData)
							{
								if((myCnt == 0) || (myCnt % 7 == 0)) 
								{
									if((uiSToken == CMND_SINGLE_CHAN_DIGIN_SCAN) && (myCnt != 0)) 
										 if(!fUSB_ERROR) printf("   %d", scanStruct.diginBuff[scanStruct.uiTailPntr]);
									if(!fUSB_ERROR) printf("\n"); fNewLineStuff = TRUE;
								}
								if((dblVolts > 0) || (dblVolts == 0)) 
								{
									if(fNewLineStuff) printf("   %lf ", dblVolts);
									else if(!fUSB_ERROR) printf("    %lf ", dblVolts); 
								}
								else 
								{
									if(fNewLineStuff) printf("  %lf ", dblVolts);
									else if(!fUSB_ERROR) printf("   %lf ", dblVolts);
								}
							}

							if(!fUSB_ERROR)
							if(fpFile)
							{
								sprintf(fBuff, "%5u ", myCnt); FILE_WRITE	
								if((dblVolts > 0) || (dblVolts == 0)) 
								{ sprintf(fBuff, "\t %lf ", dblVolts); FILE_WRITE }
								else 
								{ sprintf(fBuff, "\t%lf ", dblVolts);  FILE_WRITE }
								sprintf(fBuff, "\n"); FILE_WRITE	
							}

							if((!fContinuousScan) && (!fTimedScan))
							if(uiScanNum >= (uiScansToDo - 1))	
							{
								fScanThreadWait = TRUE;  //MUST BE BEFORE ANY BREAK OUT OF THIS MAIN LOOP!!
								fContScanDone = TRUE;
								fBreakAllLoops = TRUE;
								break;
							}

							uiScanNum++; 
							myCnt++;

							//write many small log files instead of one very large one
							if(fpFile) 
							{ 
								if((uiScanNum % 30000) == 0)
								{
									bLogFileCnt++;
									fflush(fpFile); 
									fclose(fpFile); 
									char miniBuff[255];
									char miniBuff2[25];
									strcpy(miniBuff, timeBuff);
									sprintf(miniBuff2, "_%u.txt", bLogFileCnt);
									strcat(miniBuff, miniBuff2);
									fpFile = fopen(miniBuff, "w+");
								}
							}
						}
					}

					//FOR REFERENCE:
					//#define	CMND_SINGLE_CHAN_SCAN			0x89  /* Start a SingleChan scan */        
					//#define	CMND_MULTI_CHAN_SCAN			0x8B  /* MultiChan scan */
					//#define	CMND_MULTI_CHAN_CAL_SCAN		0x8C  /* MultiCalScan */
					//#define	CMND_SINGLE_CHAN_DIGIN_SCAN		0x8D  /* SingleChanScanning /w digIn */
					//#define	CMND_MULTI_CHAN_DIGIN_SCAN		0x8E  /* MultiScan w/ digIn */
					//#define	CMND_MULTI_CHAN_CAL_DIGIN_SCAN	0x8F  /* MultiCalScan w/ digIn */

					if((uiSToken == CMND_MULTI_CHAN_SCAN) || (uiSToken == CMND_MULTI_CHAN_DIGIN_SCAN)
							|| (uiSToken == CMND_MULTI_CHAN_CAL_SCAN) || (uiSToken == CMND_MULTI_CHAN_CAL_DIGIN_SCAN))
					{
						for(bCurrentIndexForCalScan = 0; bCurrentIndexForCalScan 
							< (bTchan + bAddForOffsetChan); bCurrentIndexForCalScan++)
						{
							dblVolts = dblTempBuffForMultiChanScan[bCurrentIndexForCalScan];
							
							if((uiSToken == CMND_MULTI_CHAN_CAL_SCAN) || (uiSToken == CMND_MULTI_CHAN_CAL_DIGIN_SCAN))
							{
								if(bCurrentIndexForCalScan < (bTchan + bAddForOffsetChan) - 1)
								{ dblVolts -= dblTempBuffForMultiChanScan[(bTchan + bAddForOffsetChan) - 1]; } //subtract offset

								else
								{ }
							}
							if(dblVolts > 5) dblVolts = 5; if(dblVolts < -5) dblVolts = -5;  //sometimes offset can do this

							//start of all channels - display the scan count
							if(  (myCnt == 0) || (  (myCnt % (bTchan + bAddForOffsetChan)  ) == 0)  )	
							{
								if((!fContinuousScan) && (!fTimedScan))
								if((myCnt / scanStruct.uiScanChans) >= uiScansToDo)	
								{
									fScanThreadWait = TRUE;  //MUST BE BEFORE ANY BREAK OUT OF THIS MAIN LOOP!!
									fContScanDone = TRUE;
									fBreakAllLoops = TRUE;
									break;
								}

								if(!fUSB_ERROR && fDoDispData) printf("\n%6u", myCnt / scanStruct.uiScanChans);
							}		
							// not negative so add a space before the volts
							if(fDoDispData)
							{
								if((dblVolts > 0) || (dblVolts == 0))
								{	    
									if(fIsDigScan)printf("  %f(%2d)", dblVolts, scanStruct.diginBuff[scanStruct.uiTailPntr]);
									else if(!fUSB_ERROR) printf("  %f", dblVolts);
								}		
								else 	
								{	
									// negative so allow place for "-"						
									if(fIsDigScan)printf(" %f(%2d)", dblVolts, scanStruct.diginBuff[scanStruct.uiTailPntr]);
									else if(!fUSB_ERROR) printf(" %f", dblVolts); 
								}
							}
							
							if(!fUSB_ERROR)
							if(fpFile)
							{
								if(  (myCnt == 0) || (  (myCnt % (bTchan + bAddForOffsetChan)  ) == 0)  )	
								{
									if(!fUSB_ERROR) 
									{ sprintf(fBuff,"\n%6u", myCnt / scanStruct.uiScanChans); FILE_WRITE }
								}		
								// not negative so add a space before the volts
								if((dblVolts > 0) || (dblVolts == 0))
								{	    
									if((fIsDigScan) && (!fUSB_ERROR))
									{ sprintf(fBuff,"\t %f(%2d)", dblVolts, scanStruct.diginBuff[scanStruct.uiTailPntr]); FILE_WRITE }
									else if(!fUSB_ERROR)
									{ sprintf(fBuff,"\t %f", dblVolts); FILE_WRITE }
								}		
								else 	
								{	
									// negative so allow place for "-"						
									if(fIsDigScan)fprintf(fpFile,"\t%f(%2d)", dblVolts, scanStruct.diginBuff[scanStruct.uiTailPntr]);
									else if(!fUSB_ERROR) 
									{ sprintf(fBuff,"\t%f", dblVolts);  FILE_WRITE }
								}
							}

							if(  myCnt % (bTchan + bAddForOffsetChan) == 0   )
								uiScanNum++; 


							//write many small log files instead of one very large one
							// THIS NEEDS MODIFIED for various scan types and channels to keep file certain size for all scenarios
							if(fpFile) 
							{ 
								if((uiScanNum % 30000) == 0)
								{
									bLogFileCnt++;
									fflush(fpFile); 
									fclose(fpFile); 
									char miniBuff[255];
									char miniBuff2[25];
									strcpy(miniBuff, timeBuff);
									sprintf(miniBuff2, "_%u.txt", bLogFileCnt);
									strcat(miniBuff, miniBuff2);
									fpFile = fopen(miniBuff, "w+");
								}
							}

							myCnt++; 
						}
					}

					//always have this in place even for timed or num-scan 
					// type scans so the scan can be exited at any time.
					if(chkKbHit())
					{
						fScanThreadWait = TRUE;  //MUST BE BEFORE ANY BREAK OUT OF THIS MAIN LOOP!!
						fContScanDone = TRUE;
						fBreakAllLoops = TRUE; 
						break;
					}
				}

				if(scanStruct.fGotBuffWrap)
				{ if(!fUSB_ERROR) printf("got (LOCAL) buffer wrap \n"); }
				if((scanStruct.fScanThreadLoopDone == TRUE) && (scanStruct.uiPointsInBuff == 0))
				{ 
					break; 
				}
			}

			if(fBreakAllLoops) 
			{
				break;  
			}
		}
	}
	else scanStruct.inThread = 0;

	if(fpFile) { fflush(fpFile); fclose(fpFile); }

	printf("\nEnding scan - please wait . . .\n");

	//signal okay for scan thread in helperFuncs.c to send endscan token
	fMainScanLoopBusy = FALSE; 

	while(fScanThreadWait == TRUE) { usleep(100); fMainScanLoopBusy = FALSE; }  //4-27-17d
	while(!fEndScanTokenSent) { usleep(100); }

	printf("total scans: %u\n", uiScanNum + 1); 

	if(!fGotScanErr_TEMP) printf("Done scan \n");
	else 
	{
		printf("NOTE: scan ended due to UNKNOWN ERROR \n");
		printf("uiPointsInBuff: %u, uiTailPntr: %u, uiHeadPntr: %u \n", scanStruct.uiPointsInBuff, scanStruct.uiTailPntr, scanStruct.uiHeadPntr);
	}

	while(chkKbHit() == TRUE) 
	{ usleep(10); doGetch();}
	printf("\nuse mouse wheel (or scroll bar) to scroll up to view data or: ");
	printf("\n     <ENTER> to exit scan mode ");

	cInput = doGetch(); 

	system ("clear");
	
	if(fUSB_ERROR)
	{ fUSB_ERROR = FALSE; fSignedOn = FALSE; usDeviceID = 0; }

	return 1;
}

//#########################
//#########################
// DOGETCH ################
//#########################
//#########################
// reads from keypress, doesn't echo 
// from:   http://stackoverflow.com/questions/3276546/how-to-implement-getch-function-of-c-in-linux
int doGetch(void)
{
    struct termios oldattr, newattr;
    int ch;
    tcgetattr( STDIN_FILENO, &oldattr );
    newattr = oldattr;
    newattr.c_lflag &= ~( ICANON | ECHO );
    tcsetattr( STDIN_FILENO, TCSANOW, &newattr );
    ch = getchar();
    tcsetattr( STDIN_FILENO, TCSANOW, &oldattr );
    return ch;
}


//####################
//####################
//####################
//## CHKKBHIT ########
//####################
// check key-hit using "select()". 
// from: http://man7.org/linux/man-pages/man2/select.2.html
BOOLEAN chkKbHit(void)
{
	fd_set rfds;
	struct timeval tv;
	int retval;

	// Watch stdin (fd 0) to see when it has input. 
	FD_ZERO(&rfds);
	FD_SET(0, &rfds);

	// Wait up to five seconds - original example
	//tv.tv_sec = 5; tv.tv_usec = 0;
	tv.tv_sec = 0;
	tv.tv_usec = 10; //wait just 10 uSec

	retval = select(1, &rfds, NULL, NULL, &tv);
	// Don't rely on the value of tv now! 

	if (retval == -1)
	{
	   //perror("select()");	//original
	}
	else if (retval)
		return TRUE;
		//printf("Data is available now.\n");		//original
		//FD_ISSET(0, &rfds) will be true.			//original 
	else
		//printf("No data within five seconds.\n");	//original
		return FALSE;

	//exit(EXIT_SUCCESS);	//original
}

//library calls used from Win API
//BOOLEAN EX_ConnectOneDevice(USHORT usDevID, double dblRate, BYTE bChan);
//BOOLEAN EX_GetOneConversion ( USHORT usDevID, double* pdblMilliVolts )
//BOOLEAN EX_SendChan(USHORT usDevID, BYTE bChan, BYTE* pbLastDigin)
//BOOLEAN EX_SendDAC(USHORT usDevID, DOUBLE dblVoltage, BYTE bDAC)
//BOOLEAN EX_GetCalculatedRate ( USHORT usDevID , double* pdblRate )
