//NOTE: this is building of, and running sampAppBig ONLY
/*
in app folder: LL_USB_min.h, LL_USB_min.c
min app compile:  
	sudo gcc -o LL_USB_min LL_USB_min.c -lLL_USB30x  -lm -pthread &>makeout
run the app:
	sudo ./LL_USB_min

NOTE: add "-Wall" after "gcc" to show "all" warnings. Without, only all errors and "some" warnings are shown.
*/
// NOTE: many variables declared as "volatile" as they are shared between 
// scan thread and and other process
#ifndef MAIN_MIN_H
#define MAIN_MIN_H 
//following needed to prevent compilation error related to: pthread_mutex_lock
// used to prevent microcode (firmware) buffer wrap when scanning.
#define _GNU_SOURCE
 
#include <string.h>
#include <stdlib.h>

#include <stdio.h>
#include <sys/types.h>
#include <string.h> 
#include <pthread.h>
#include <unistd.h> 
#include <time.h>

#define BOOLEAN unsigned char
#define BYTE unsigned char
#define UINT unsigned int
#define INT int
#define SHORT signed short
#define USHORT unsigned short
#define DOUBLE double
#define FALSE 0
#define TRUE 1


typedef struct _SCAN_STRUCT
{
	//set if the buffer in scan thread in helperFuncs.c has gotten more bytes than the 
	// size of the buffer - 
	volatile BYTE fGotBuffWrap;
	//set in helperFuncs.c when scan thread has successfully been entered. Just because
	// an attempt is made to begin the thread in scanFuncs.c, it may not actually be entered
	// if there aren't enough resources, or for some other reason.
	volatile int inThread;
	//set in helperFuncs.c when scan loop is exited (but thread still active)
	volatile int isScanning;
	//set in helperFuncs.c when scan thread loop has exited
	volatile BOOLEAN fScanThreadLoopDone; 
	int* scanBuff;		//type "int" because it can be positive or negative "raw" data
	BYTE* diginBuff; 
	
	//this allows for low-level stacking of scans which allows for slightly 
	// more time for user-type apps to have a time-slice. In other words, the
	// kernel-mode threading process can be reading data from the firmware while
	// user mode apps are doing things and then the app can start processing that
	// data when it gets a slice.
	volatile UINT uiStackedScans; //MUST ALWAYS BE AT LEAST 1 (set in main())
	volatile UINT uiHeadPntr;
	volatile UINT uiTailPntr;
	volatile UINT uiBufferSizeMin;
	volatile UINT uiBufferSizeMax;
//these are codes used by the firmware to determine the scan type to do
// single chan regular token
#define	CMND_SINGLE_CHAN_SCAN			0x89 
// single chan with digin token
#define	CMND_SINGLE_CHAN_DIGIN_SCAN		0x8D  
// multi chan regular token
#define	CMND_MULTI_CHAN_SCAN			0x8B 
// multi chan with cal token
#define	CMND_MULTI_CHAN_CAL_SCAN		0x8C 
// multi chan with digin token	
#define	CMND_MULTI_CHAN_DIGIN_SCAN		0x8E 
// multi chan with cal and digin token
#define	CMND_MULTI_CHAN_CAL_DIGIN_SCAN	0x8F 

// endscan token
#define	EndScan							0x8A           
	volatile UINT uiScanToken;
	volatile UINT uiScanChans;  
	volatile UINT uiPointsInBuff;
} SCAN_STRUCT, *PSCAN_STRUCT;


//shared variables
//this is buffer used for strings to be written to the screen. Since 
// the longest string would be 80 characters it's set to the size shown.
//char scrnBuff[81];

BYTE bDBG;
BYTE bSingleChanScanChan;
BOOLEAN fGotDigin;
BYTE bLastDigin;
BOOLEAN fSignedOn;

pthread_t threadID;

BOOLEAN setDefaultVars(void);
BOOLEAN Scan(void);

BOOLEAN chkKbHit(void);
int doGetch(void);
 

//This flag used to tell the scan menu to update it's screen
// based on the scan type that has been selected from nothing
// having been selected
BOOLEAN fCurrentScanTypeChanged;

//This flag is used when the scan thread has exited it's loop, is sending the
// end-scan token and waiting for the echoed bytes to keep the scanning function
// in helperFuncs.c from exiting or doing anything before those results are received.
//When they are received, the scan thread sets the flag to FALSE, exits, and the
// scanning funtion in scanFuncs.c displays a message for user to press the <ENTER>
// to go back to scanMenu.  
extern volatile BOOLEAN fScanThreadWait;   

//set to TRUE when scan thread is entered. This is used to let the scan function
// in scanFuncs know if the thread in helperFuncs even started, which on rare
// occasions in may not, due perhaps to something like lack of resources
extern volatile BOOLEAN fScanThreadActive;   

//set in scanMenu for continuous scan - overides all other scan-length options
extern volatile BOOLEAN fContinuousScan; 

//set TRUE in scanFuncs.c if key-hit is detected, checked 
// in helperFuncs.c which exits it's data-loop if TRUE
// any of the scans will end their loops if a key is pressed
extern volatile BOOLEAN fContScanDone;	

//sometimes the main scan-data-taking loop in thread in helperFuncs.c may exit before
// the loop data-display loop in helperFuncs.c. This flag is used to prevent it from 
// displaying exit related prints to the screen while the data-display loop is still
// displaying data.
extern volatile BOOLEAN fMainScanLoopBusy;	 

//this flag is set after the thread has sent the end-scan token so the scan function
// in scanFuncs.c knows it can display the text for the user to press <ENTER> to return
// the scan menu
extern volatile BOOLEAN fEndScanTokenSent;	 

//this flag is set in scan thread in helperFuncs.c based on either transfer->status
// which could likely indicate a USB device disconnect, or a lengthy wait for scan
// data to be returned indicating a possible power cable disconnect.
extern volatile BOOLEAN fUSB_ERROR;	 

//set in scan menu if log-scan is chosen
extern volatile BOOLEAN fDoScanLog;	 
extern volatile BOOLEAN fDoDispData;	 

struct libusb_device_handle *hDevice_global; 

//used in initUsbLib() in helperFuncs.c. Was local, made global in case
// it should be needed elsewhere sometime
struct libusb_device **devs;

//error handling flags used to communicate between process in scanFuncs.c and scan thread
// in helperFuncs.c. One set if it appears the power supply to the model 30x has been
// removed and the other in case a USB cable disconnect (or other unknown error) seems
// to have occurred.
extern volatile volatile BOOLEAN fPossibleScanPower_Err;
extern volatile BOOLEAN fPossibleScanUSB_Err;

//set in scanMenu if option chosen to do certain number of scans
extern volatile UINT uiScansToDo; 


//4-27-17n
UINT uiConsoleStartWidth;
UINT uiConsoleStartHeight;
BOOLEAN fConsoleSizeChanged;


//set in scanMenu if option chosen to do a scan that lasts a particular
//length of time. The seconds, minutes, and hours cannot be combined - only
// one is active at a time dependent on it's flag being set.
extern volatile BOOLEAN fTimedScan;
extern volatile UINT uiScanHours; 
extern volatile BOOLEAN fScanHours;
extern volatile UINT uiScanMinutes; 
extern volatile BOOLEAN fScanMinutes;
extern volatile UINT uiScanSeconds; 
extern volatile BOOLEAN fScanSeconds; 

//6-16-17
double dblDevActualRate;


USHORT usDeviceID;

int iErr;
int iRetCode;

double dblRate;
double dblVolts;


//###########################################
//## EXPORTABLE functions ###################
extern BOOLEAN EX_Connect ( USHORT* pDevID );
//preferred used in Windows VB graphic code
extern BOOLEAN EX_ConnectOneDevice(USHORT usDevID, double dblRate, BYTE bChan);
//maybe not use this one
extern BOOLEAN EX_ConnectAllDevices ( BYTE* pbNumDevices, USHORT* pDevList, double* pdblRates ); 
extern BOOLEAN EX_GetOneConversion (USHORT usDevID, double* pdblMilliVolts);
extern BOOLEAN EX_SendChan(USHORT usDevID, BYTE bChan, BYTE* pbLastDigin);
extern BOOLEAN EX_SendDAC(USHORT usDevID, DOUBLE dblVoltage, BYTE bDAC);
extern BOOLEAN EX_GetCalculatedRate (USHORT usDevID , double* pdblRate);
extern BOOLEAN EX_SendRate(USHORT usDevID, double* dblRate);
extern BOOLEAN EX_SendDigout(USHORT usDevID, BYTE bDigOut);
extern BOOLEAN EX_GetDigin(USHORT usDevID, BYTE* bDigIn);
extern BOOLEAN EX_SetScanType (USHORT usDevID, BYTE bScanType);
extern BOOLEAN EX_SetDataLogOptions (USHORT usDevID, UINT uiDataLogMethod, UINT uiLogFileMaxSize, UINT uiMisc);
extern BOOLEAN EX_SetMultiChanScanChans (USHORT usDevID, BYTE bChanCnt);
extern BOOLEAN EX_Run(USHORT usDevID);
extern BOOLEAN EX_CheckScanStatus (USHORT usDevID, UINT* puiNumScansRemainingInBuff, BOOLEAN* pfStillScanning, UINT* puiStatusCode);
extern BOOLEAN EX_GetScanDataDbl (USHORT usDevID, double* pdblDataBuff, UINT uiNumScansToReturn, BOOLEAN fGetAllRequested,
        UINT* puiNumScansReturned, UINT* puiNumScansRemainingInBuff, BOOLEAN* pfStillScanning, UINT* puiTotalBuffSize, UINT* puiStatusCode);
extern BOOLEAN EX_GetScanDataDblDigin (USHORT usDevID, double* pdblDataBuff, BYTE* pbDiginBuff,
                                UINT uiNumScansToReturn, BOOLEAN fGetAllRequested, UINT* puiNumScansReturned,
                                UINT* puiNumScansRemainingInBuff, BOOLEAN* pfStillScanning, UINT* puiTotalBuffSize, UINT* puiErrCode);
extern BOOLEAN EX_EndScan(USHORT usDevID);
extern BOOLEAN EX_SetLengthOfRunSec (USHORT usDevID, UINT uiSeconds);
extern BOOLEAN EX_RunWithMods(USHORT usDevID, BYTE bScanType, BYTE bChanCnt, BYTE bDataLogOptions, BYTE bLengthOfRun);
extern void* doScan(void *arg);
//###########################################
//###########################################


//from test.c
//local functions
int doGetch(void);
int kbhit(BOOLEAN fIncDec);


//END OF: #define MAIN_MIN_H
#endif





